@@ -1,4 +1,6 @@
 #![no_std]
+#![allow(clippy::inline_always)]
+#![allow(clippy::doc_markdown)]
 #![allow(incomplete_features)]
 #![feature(const_fn)]
 #![feature(const_generics)]
@@ -12,6 +14,7 @@
 #![feature(maybe_uninit_ref)]
 #![feature(maybe_uninit_uninit_array)]
 #![cfg_attr(feature = "std", feature(read_initializer))]
+#![feature(slice_from_raw_parts)]
 #![feature(slice_partition_dedup)]
 #![feature(specialization)]
 #![feature(trusted_len)]
@@ -23,10 +26,11 @@ use crate::utils::reverse_copy;
 use core::cmp::{Ord, PartialEq};
 use core::intrinsics;
 use core::marker::PhantomData;
-use core::mem::{self, MaybeUninit};
-use core::ops::{Bound::Excluded, Bound::Included, Bound::Unbounded, RangeBounds};
+use core::mem::MaybeUninit;
+use core::ops::{
+  Add, Bound::Excluded, Bound::Included, Bound::Unbounded, Div, Mul, RangeBounds, Sub,
+};
 use core::ptr;
-use core::slice;
 
 #[cfg(any(feature = "std", rustdoc))]
 extern crate alloc;
@@ -92,9 +96,12 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// any contents after that point are ignored.
   ///
   /// The `N2` parameter does not need to be provided explicitly, and can be inferred from the array
-  /// itself. This function does *not* leak memory, as any ignored extra elements in the source
-  /// array are explicitly dropped with [`drop_in_place`](core::ptr::drop_in_place) before
-  /// [`forget`](core::mem::forget) is called on it.
+  /// itself.
+  ///
+  /// This function does *not* leak memory, as any ignored extra elements in the source
+  /// array are explicitly dropped with [`drop_in_place`](core::ptr::drop_in_place) after it is
+  /// first wrapped in an instance of [`MaybeUninit`](core::mem::MaybeUninit) to inhibit the
+  /// automatic calling of any destructors its contents may have.
   ///
   /// Example usage:
   /// ```
@@ -113,7 +120,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// let v = StaticVec::from(["A", "B", "C", "D"]);
   /// ```
   #[inline]
-  pub fn new_from_array<const N2: usize>(mut values: [T; N2]) -> Self {
+  pub fn new_from_array<const N2: usize>(values: [T; N2]) -> Self {
     if N == N2 {
       Self::from(values)
     } else {
@@ -124,9 +131,10 @@ impl<T, const N: usize> StaticVec<T, N> {
             values
               .as_ptr()
               .copy_to_nonoverlapping(data.as_mut_ptr() as *mut T, N2.min(N));
-            // Drops any extra values left in the source array, then "forgets it".
-            ptr::drop_in_place(values.get_unchecked_mut(N2.min(N)..N2));
-            mem::forget_unsized(values);
+            // Wrap the values in a MaybeUninit to inhibit their destructors (if any),
+            // then manually drop any excess ones.
+            let mut forgotten = MaybeUninit::new(values);
+            ptr::drop_in_place(forgotten.get_mut().get_unchecked_mut(N2.min(N)..N2));
             data
           }
         },
@@ -262,14 +270,14 @@ impl<T, const N: usize> StaticVec<T, N> {
   pub fn as_slice(&self) -> &[T] {
     // Safety: `self.as_ptr()` is a pointer to an array for which the first `length`
     // elements are guaranteed to be initialized. Therefore this is a valid slice.
-    unsafe { slice::from_raw_parts(self.as_ptr(), self.length) }
+    unsafe { &*ptr::slice_from_raw_parts(self.as_ptr(), self.length) }
   }
 
   /// Returns a mutable reference to a slice of the StaticVec's inhabited area.
   #[inline(always)]
   pub fn as_mut_slice(&mut self) -> &mut [T] {
     // Safety: See as_slice.
-    unsafe { slice::from_raw_parts_mut(self.as_mut_ptr(), self.length) }
+    unsafe { &mut *ptr::slice_from_raw_parts_mut(self.as_mut_ptr(), self.length) }
   }
 
   /// Returns a constant reference to the element of the StaticVec at `index`,
@@ -761,6 +769,41 @@ impl<T, const N: usize> StaticVec<T, N> {
     self.length += item_count;
   }
 
+  /// Returns a new StaticVec consisting of the elements of `self` and `other` concatenated in
+  /// linear fashion such that the first element of `other` comes immediately after the last
+  /// element of `self`.
+  ///
+  /// The `N2` parameter does not need to be provided explicitly, and can be inferred directly from
+  /// the constant `N2` constraint of `other` (which may or may not be the same as the `N`
+  /// constraint of `self`.)
+  ///
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to
+  /// avoid soundness issues and also allow for a more efficient implementation than would otherwise
+  /// be possible.
+  ///
+  /// Example usage:
+  /// ```
+  /// assert_eq!(
+  ///  staticvec!["A, B"].concat(&staticvec!["C", "D", "E", "F"]),
+  ///  ["A, B", "C", "D", "E", "F"]
+  /// );
+  /// ```
+  #[inline]
+  pub fn concat<const N2: usize>(&self, other: &StaticVec<T, N2>) -> StaticVec<T, { N + N2 }>
+  where T: Copy {
+    let mut res = StaticVec::new();
+    unsafe {
+      self
+        .as_ptr()
+        .copy_to_nonoverlapping(res.as_mut_ptr(), self.length);
+      other
+        .as_ptr()
+        .copy_to_nonoverlapping(res.mut_ptr_at_unchecked(self.length), other.length);
+    }
+    res.length = self.length + other.length;
+    res
+  }
+
   /// Returns a [`Vec`](alloc::vec::Vec) containing the contents of the StaticVec instance.
   /// The returned [`Vec`](alloc::vec::Vec) will initially have the same value for
   /// [`len`](alloc::vec::Vec::len) and [`capacity`](alloc::vec::Vec::capacity) as the source
@@ -939,12 +982,9 @@ impl<T, const N: usize> StaticVec<T, N> {
     for left in self {
       let mut found = false;
       for right in other {
-        match left == right {
-          false => (),
-          true => {
-            found = true;
-            break;
-          }
+        if left == right {
+          found = true;
+          break;
         }
       }
       if !found {
@@ -983,12 +1023,9 @@ impl<T, const N: usize> StaticVec<T, N> {
     for left in self {
       let mut found = false;
       for right in other {
-        match left == right {
-          false => (),
-          true => {
-            found = true;
-            break;
-          }
+        if left == right {
+          found = true;
+          break;
         }
       }
       if !found {
@@ -998,12 +1035,9 @@ impl<T, const N: usize> StaticVec<T, N> {
     for right in other {
       let mut found = false;
       for left in self {
-        match right == left {
-          false => (),
-          true => {
-            found = true;
-            break;
-          }
+        if right == left {
+          found = true;
+          break;
         }
       }
       if !found {
@@ -1029,6 +1063,138 @@ impl<T, const N: usize> StaticVec<T, N> {
     (self.as_mut_ptr(), self.length, N)
   }
 
+  /// Linearly adds (in a mathematical sense) the contents of two same-capacity
+  /// StaticVecs and returns the results in a new one of equal capacity.
+  ///
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to allow
+  /// for an efficient implementation, and [`Add`](core::ops::Add) to make it possible
+  /// to add the elements.
+  ///
+  /// For both performance and safety reasons, this function requires that both `self`
+  /// and `other` are at full capacity, and will panic if that is not the case (that is,
+  /// if `self.is_full() && other.is_full()` is not equal to `true`.)
+  ///
+  /// Example usage:
+  /// ```
+  /// const A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
+  /// const B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
+  /// assert_eq!(A.added(&B), [6.0, 8.0, 10.0, 12.0]);
+  /// ```
+  #[inline(always)]
+  pub fn added(&self, other: &Self) -> Self
+  where T: Copy + Add<Output = T> {
+    assert!(self.is_full() && other.is_full());
+    let mut res = Self::new();
+    for i in 0..N {
+      unsafe {
+        res
+          .mut_ptr_at_unchecked(i)
+          .write(*self.get_unchecked(i) + *other.get_unchecked(i));
+      }
+    }
+    res.length = N;
+    res
+  }
+
+  /// Linearly subtracts (in a mathematical sense) the contents of two same-capacity
+  /// StaticVecs and returns the results in a new one of equal capacity.
+  ///
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to allow
+  /// for an efficient implementation, and [`Sub`](core::ops::Sub) to make it possible
+  /// to subtract the elements.
+  ///
+  /// For both performance and safety reasons, this function requires that both `self`
+  /// and `other` are at full capacity, and will panic if that is not the case (that is,
+  /// if `self.is_full() && other.is_full()` is not equal to `true`.)
+  ///
+  /// Example usage:
+  /// ```
+  /// const A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
+  /// const B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
+  /// assert_eq!(A.subtracted(&B), [2.0, 2.0, 2.0, 2.0]);
+  /// ```
+  #[inline(always)]
+  pub fn subtracted(&self, other: &Self) -> Self
+  where T: Copy + Sub<Output = T> {
+    assert!(self.is_full() && other.is_full());
+    let mut res = Self::new();
+    for i in 0..N {
+      unsafe {
+        res
+          .mut_ptr_at_unchecked(i)
+          .write(*self.get_unchecked(i) - *other.get_unchecked(i));
+      }
+    }
+    res.length = N;
+    res
+  }
+
+  /// Linearly multiplies (in a mathematical sense) the contents of two same-capacity
+  /// StaticVecs and returns the results in a new one of equal capacity.
+  ///
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to allow
+  /// for an efficient implementation, and [`Mul`](core::ops::Mul) to make it possible
+  /// to multiply the elements.
+  ///
+  /// For both performance and safety reasons, this function requires that both `self`
+  /// and `other` are at full capacity, and will panic if that is not the case (that is,
+  /// if `self.is_full() && other.is_full()` is not equal to `true`.)
+  ///
+  /// Example usage:
+  /// ```
+  /// const A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
+  /// const B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
+  /// assert_eq!(A.multiplied(&B), [8.0, 15.0, 24.0, 35.0]);
+  /// ```
+  #[inline(always)]
+  pub fn multiplied(&self, other: &Self) -> Self
+  where T: Copy + Mul<Output = T> {
+    assert!(self.is_full() && other.is_full());
+    let mut res = Self::new();
+    for i in 0..N {
+      unsafe {
+        res
+          .mut_ptr_at_unchecked(i)
+          .write(*self.get_unchecked(i) * *other.get_unchecked(i));
+      }
+    }
+    res.length = N;
+    res
+  }
+
+  /// Linearly divides (in a mathematical sense) the contents of two same-capacity
+  /// StaticVecs and returns the results in a new one of equal capacity.
+  ///
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to allow
+  /// for an efficient implementation, and [`Div`](core::ops::Div) to make it possible
+  /// to divide the elements.
+  ///
+  /// For both performance and safety reasons, this function requires that both `self`
+  /// and `other` are at full capacity, and will panic if that is not the case (that is,
+  /// if `self.is_full() && other.is_full()` is not equal to `true`.)
+  ///
+  /// Example usage:
+  /// ```
+  /// const A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
+  /// const B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
+  /// assert_eq!(A.divided(&B), [2.0, 1.6666666666666667, 1.5, 1.4]);
+  /// ```
+  #[inline(always)]
+  pub fn divided(&self, other: &Self) -> Self
+  where T: Copy + Div<Output = T> {
+    assert!(self.is_full() && other.is_full());
+    let mut res = Self::new();
+    for i in 0..N {
+      unsafe {
+        res
+          .mut_ptr_at_unchecked(i)
+          .write(*self.get_unchecked(i) / *other.get_unchecked(i));
+      }
+    }
+    res.length = N;
+    res
+  }
+
   #[doc(hidden)]
   #[inline(always)]
   pub(crate) const fn new_data_uninit() -> MaybeUninit<[T; N]> {
