@@ -5,6 +5,7 @@ use staticvec::*;
 
 use core::cell;
 
+#[cfg(not(miri))]
 #[cfg(feature = "std")]
 use std::panic::{self, AssertUnwindSafe};
 
@@ -207,6 +208,7 @@ fn clone_from_longer() {
   assert_eq!(dst, src);
 }
 
+#[cfg(not(miri))]
 #[cfg(feature = "std")]
 #[test]
 fn panicking_clone() {
@@ -277,6 +279,18 @@ fn panicking_clone() {
   assert_eq!(lifespan_tracker.drop_count(), 11);
 }
 
+#[test]
+fn concat() {
+  assert_eq!(
+    staticvec!["A, B"].concat(&staticvec!["C", "D", "E", "F"]),
+    ["A, B", "C", "D", "E", "F"]
+  );
+  let v = StaticVec::<i32, 0>::from([]).concat(&StaticVec::<i32, 0>::from([]));
+  assert_eq!(v, []);
+  let v2 = staticvec![4, 5, 6].concat(&staticvec![1, 2, 3]);
+  assert_eq!(v2, [4, 5, 6, 1, 2, 3]);
+}
+
 #[test]
 fn dedup() {
   let mut vec = staticvec![1, 2, 2, 3, 2];
@@ -570,6 +584,16 @@ fn macros() {
   assert!(V4 == [27; 128]);
 }
 
+#[test]
+fn math_functions() {
+  static A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
+  static B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
+  assert_eq!(A.added(&B), [6.0, 8.0, 10.0, 12.0]);
+  assert_eq!(A.subtracted(&B), [2.0, 2.0, 2.0, 2.0]);
+  assert_eq!(A.multiplied(&B), [8.0, 15.0, 24.0, 35.0]);
+  assert_eq!(A.divided(&B), [2.0, 1.6666666666666667, 1.5, 1.4]);
+}
+
 #[test]
 fn mut_ptr_at() {
   let mut v = staticvec![1, 2, 3];
@@ -606,15 +630,17 @@ fn new_from_array() {
   assert_eq!(v, [1, 2, 3]);
   let v2 = StaticVec::<i32, 3>::new_from_array([1, 2, 3, 4, 5, 6]);
   assert_eq!(v2, [1, 2, 3]);
-  // Miri gives a false positive "dangling reference" for this.
-  #[cfg(not(miri))]
-  let v5 = StaticVec::<Box<Struct>, 1>::new_from_array([
+  let v5 = StaticVec::<Box<Struct>, 2>::new_from_array([
     Box::new(Struct { s: "AAA" }),
     Box::new(Struct { s: "BBB" }),
     Box::new(Struct { s: "CCC" }),
+    Box::new(Struct { s: "DDD" }),
+    Box::new(Struct { s: "EEE" }),
   ]);
-  #[cfg(not(miri))]
-  assert_eq!(v5, [Box::new(Struct { s: "AAA" })]);
+  assert_eq!(
+    v5,
+    [Box::new(Struct { s: "AAA" }), Box::new(Struct { s: "BBB" })]
+  );
 }
 
 #[test]
